// Esse programa l um arquivo de acesso aleatrio seqencialmente, atualiza
// os dados previamente gravados no arquivo, cria dados a ser colocados
// no arquivo e exclui os dados previamente no arquivo.
#include <iostream>
#include <fstream> 
#include <iomanip>
#include <cstdlib> 
using namespace std; 

#include "ClientData.h" // definio da classe ClientData

int enterChoice();
void createTextFile( fstream& );
void updateRecord( fstream& );
void newRecord( fstream& );
void deleteRecord( fstream& );
void outputLine( ostream&, const ClientData & );
int getAccount( const char * const );

enum Choices { PRINT = 1, UPDATE, NEW, DELETE, END };

int main()
{
   // abre o arquivo para leitura e gravao              
   fstream inOutCredit( "credit.dat", ios::in | ios::out );

   // fecha programa se fstream no puder abrir o arquivo
   if ( !inOutCredit ) 
   {
      cerr << "File could not be opened." << endl;
      exit ( 1 );
   } // fim do if
   
   int choice; // armazena a escolha do usurio 

   // permite ao usurio especificar a ao
   while ( ( choice = enterChoice() ) != END ) 
   {
      switch ( choice ) 
      {
         case PRINT: // cria arquivo de texto a partir do arquivo de registro
            createTextFile( inOutCredit );
            break;
         case UPDATE: // atualiza o registro
            updateRecord( inOutCredit );
            break;
         case NEW: // cria registro
            newRecord( inOutCredit );
            break;
         case DELETE: // exclui registro existente
            deleteRecord( inOutCredit );
            break;
         default: // exibe erro se o usurio no selecionar uma escolha vlida
            cerr << "Incorrect choice" << endl;
            break;
      } // fim do switch

      inOutCredit.clear(); // reinicializa indicador de fim do arquivo 
   } // fim do while

   return 0;
} // fim de main

// permite ao usurio inserir escolhas de menu
int enterChoice()
{
   // exibe opes disponveis
   cout << "\nEnter your choice" << endl
      << "1 - store a formatted text file of accounts" << endl
      << "    called \"print.txt\" for printing" << endl
      << "2 - update an account" << endl
      << "3 - add a new account" << endl
      << "4 - delete an account" << endl
      << "5 - end program\n? ";

   int menuChoice;
   cin >> menuChoice; // insere a seleo de menu do usurio
   return menuChoice;
} // fim da funo enterChoice

// cria arquivo de texto formatado para impresso
void createTextFile( fstream &readFromFile )
{
   // cria arquivo de texto                      
   ofstream outPrintFile( "print.txt", ios::out );

   // fecha o programa se ofstream no criar arquivo
   if ( !outPrintFile ) 
   {
      cerr << "File could not be created." << endl;
      exit( 1 );
   } // fim do if

   outPrintFile << left << setw( 10 ) << "Account" << setw( 16 )
      << "Last Name" << setw( 11 ) << "First Name" << right    
      << setw( 10 ) << "Balance" << endl;                      

   // configura o ponteiro de posio de arquivo para o comeo de readFromFile
   readFromFile.seekg( 0 );                                                  

   // l o primeiro registro a partir do arquivo de registro
   ClientData client;
   readFromFile.read( reinterpret_cast< char * >( &client ),
      sizeof( ClientData ) );                              

   // copia todos os registros a partir do arquivo de registro para um arquivo de texto
   while ( !readFromFile.eof() ) 
   {
      // grava um nico registro no arquivo de texto
      if ( client.getAccountNumber() != 0 ) // pula registros vazios
         outputLine( outPrintFile, client );

      // l o prximo registro a partir do arquivo de registro  
      readFromFile.read( reinterpret_cast< char * >( &client ),
         sizeof( ClientData ) );                              
   } // fim do while
} // fim da funo createTextFile

// atualiza o saldo no registro
void updateRecord( fstream &updateFile )
{
   // obtm o nmero de conta a atualizar
   int accountNumber = getAccount( "Enter account to update" );

   // move o ponteiro de posio de arquivo para corrigir o registro no arquivo
   updateFile.seekg( ( accountNumber - 1 ) * sizeof( ClientData ) );      

   // l o primeiro registro do arquivo
   ClientData client;
   updateFile.read( reinterpret_cast< char * >( &client ),
      sizeof( ClientData ) );                            

   // atualiza o registro
   if ( client.getAccountNumber() != 0 ) 
   {
      outputLine( cout, client ); // exibe o registro

      // solicita para o usurio especificar a transao
      cout << "\nEnter charge (+) or payment (-): ";
      double transaction; // cobrana ou pagamento
      cin >> transaction;

      // atualiza o saldo do registro 
      double oldBalance = client.getBalance();
      client.setBalance( oldBalance + transaction );
      outputLine( cout, client ); // exibe o registro

      // move ponteiro de posio de arquivo para corrigir o registro no arquivo          
      updateFile.seekp( ( accountNumber - 1 ) * sizeof( ClientData ) );      

      // grava o registro atualizado sobre o registro antigo no arquivo
      updateFile.write( reinterpret_cast< const char * >( &client ),
         sizeof( ClientData ) );                                    
   } // fim do if
   else // exibe erro se a conta no existir
      cerr << "Account #" << accountNumber 
         << " has no information." << endl;
} // fim da funo updateRecord

// cria e insere o registro
void newRecord( fstream &insertInFile )
{
   // obtm o nmero da conta a ser criada
   int accountNumber = getAccount( "Enter new account number" );

   // move o ponteiro de posio de arquivo para corrigir o registro no arquivo
   insertInFile.seekg( ( accountNumber - 1 ) * sizeof( ClientData ) );

   // l o registro no arquivo
   ClientData client;
   insertInFile.read( reinterpret_cast< char * >( &client ),
      sizeof( ClientData ) );                              

   // cria o registro, se ele ainda no existir
   if ( client.getAccountNumber() == 0 ) 
   {
      char lastName[ 15 ];
      char firstName[ 10 ];
      double balance;

      // o usurio insere o sobrenome, o nome e o saldo
      cout << "Enter lastname, firstname, balance\n? ";
      cin >> setw( 15 ) >> lastName;
      cin >> setw( 10 ) >> firstName;
      cin >> balance;

      // utiliza os valores para preencher os valores de conta
      client.setLastName( lastName );
      client.setFirstName( firstName );
      client.setBalance( balance );
      client.setAccountNumber( accountNumber );

      // move o ponteiro de posio de arquivo para corrigir o registro no arquivo
      insertInFile.seekp( ( accountNumber - 1 ) * sizeof( ClientData ) );    

      // insere o registro no arquivo                                  
      insertInFile.write( reinterpret_cast< const char * >( &client ),
         sizeof( ClientData ) );                                     
   } // fim do if
   else // exibe erro se a conta j existir
      cerr << "Account #" << accountNumber
         << " already contains information." << endl;
} // fim da funo newRecord

// exclui um registro existente
void deleteRecord( fstream &deleteFromFile )
{
   // obtm o nmero de conta a excluir
   int accountNumber = getAccount( "Enter account to delete" );

   // move ponteiro de posio de arquivo para corrigir registro no arquivo
   deleteFromFile.seekg( ( accountNumber - 1 ) * sizeof( ClientData ) );  

   // l o registro no arquivo
   ClientData client;
   deleteFromFile.read( reinterpret_cast< char * >( &client ),
      sizeof( ClientData ) );                                

   // exclui o registro, se existir no arquivo
   if ( client.getAccountNumber() != 0 ) 
   {
      ClientData blankClient; // cria o registro em branco

      // move o ponteiro de posio de arquivo para corrigir o registro no arquivo
      deleteFromFile.seekp( ( accountNumber - 1 ) *                          
         sizeof( ClientData ) );                                             

      // substitui registro existente pelo registro em branco
      deleteFromFile.write(                                 
         reinterpret_cast< const char * >( &blankClient ),  
         sizeof( ClientData ) );                            
 
      cout << "Account #" << accountNumber << " deleted.\n";
   } // fim do if
   else // exibe erro se o registro no existir
      cerr << "Account #" << accountNumber << " is empty.\n";
} // fim de deleteRecord

// exibe um nico registro
void outputLine( ostream &output, const ClientData &record )
{
   output << left << setw( 10 ) << record.getAccountNumber()
      << setw( 16 ) << record.getLastName()
      << setw( 11 ) << record.getFirstName()
      << setw( 10 ) << setprecision( 2 ) << right << fixed 
      << showpoint << record.getBalance() << endl;
} // fim da funo outputLine

// obtm o valor do nmero da conta a partir do usurio
int getAccount( const char * const prompt )
{
   int accountNumber;

   // obtm valor do nmero da conta
   do 
   {
      cout << prompt << " (1 - 100): ";
      cin >> accountNumber;
   } while ( accountNumber < 1 || accountNumber > 100 );

   return accountNumber;
} // fim da funo getAccount



